-----------------------------------------------------------------------------------------------
-- Client Lua Script for SprintMeterPlus
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "HazardsLib"


-----------------------------------------------------------------------------------------------
-- SprintMeterPlus Module Definition
-----------------------------------------------------------------------------------------------
local SprintMeterPlus = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local Version = 120
local slimWidth = 250
local normalWidth = 250
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SprintMeterPlus:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function SprintMeterPlus:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- SprintMeterPlus OnLoad
-----------------------------------------------------------------------------------------------
function SprintMeterPlus:OnLoad()
--     load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SprintMeterPlus.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SprintMeterPlus OnDocLoaded
-----------------------------------------------------------------------------------------------
function SprintMeterPlus:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		Apollo.LoadSprites("SMPSprites.xml")
	    self.PopUp = Apollo.LoadForm(self.xmlDoc, "PopUp", FixedHudStratum, self)

		local bDefaultSettings = false		
		if self.tSettings == nil then
			self.tSettings = {}
			self.tSettings.fScale = 1
			self.tSettings.fWidth = 1
			self.tSettings.Style = "SprintMeter"
			bDefaultSettings = true
		    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SprintMeterForm", FixedHudStratumLow, self)
		else
			if self.tSettings.Style == "SprintMeter" or self.tSettings.Style == "Normal" then
			    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SprintMeterForm", FixedHudStratumLow, self)
			elseif self.tSettings.Style == "SlimMeter" or self.tSettings.Style == "Slim" then
			    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SlimMeterForm", FixedHudStratumLow, self)
			elseif self.tSettings.Style == "KuronaMeter" or self.tSettings.Style == "Kurona" then
			    self.wndMain = Apollo.LoadForm(self.xmlDoc, "KuronaMeterForm", FixedHudStratumLow, self)
			else
				self.tSettings = {}
				self.tSettings.fScale = 1
				self.tSettings.fWidth = 1
				self.tSettings.Style = "SprintMeter"
				bDefaultSettings = true
			    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SprintMeterForm", FixedHudStratumLow, self)
			end

			local nHeight = self.wndMain:GetHeight()
			local nWidth = self.wndMain:GetWidth()
		
			self.wndMain:SetAnchorOffsets(self.tSettings.cSprintMeterWindowL,self.tSettings.cSprintMeterWindowT,self.tSettings.cSprintMeterWindowL + 250 *self.tSettings.fWidth,self.tSettings.cSprintMeterWindowT + nHeight)
		end		

    self.wndMain:Show(false, true)

	self.nBreath = 100	
	self.HazardWindow = self.wndMain:FindChild("HazardWindow")

	if self.wndMain == nil then
		Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
		return
	end

	if bDefaultSettings then
		self:OnSprintMeterPlusOn()
end
		
			self.wndMain:SetScale(self.tSettings.fScale)
			self.PopUp:FindChild("MeterScaleValue"):SetText(self.tSettings.fScale)
			self.PopUp:FindChild("ScaleSliderBar"):SetValue(self.tSettings.fScale*100)
			
			self.PopUp:FindChild("WidthScaleValue"):SetText(self.tSettings.fWidth)
			self.PopUp:FindChild("WidthSliderBar"):SetValue(self.tSettings.fWidth*100)
		

		
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("smp", "OnSprintMeterPlusOn", self)
	    Apollo.RegisterEventHandler("VarChange_FrameCount", "OnFrame", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("SprintMeterPlusMenuClick", "OnSprintMeterPlusMenuClick", self)
		Apollo.RegisterTimerHandler("SprintMeterGracePeriod", "OnSprintMeterGracePeriod", self)
		Apollo.RegisterEventHandler("BreathChanged",			"OnBreathChanged", self)



		-- Do additional Addon initialization here
--			self.wndMain:FindChild("Dash1Lit"):Show(false)
--			self.wndMain:FindChild("Dash2Lit"):Show(false)
			self.SliderButton=	self.PopUp:FindChild("LeftButton")
			self.OptionsButton=	self.PopUp:FindChild("RightButton")
			self.OptionsButton:SetCheck(true)	
			
			
			self.LastRun =0
			self.LastEvade1 = 0
			self.LastEvade2 = 0
			self.bNotActive= true
			self:SetupFrameAlias()


	end
end


function SprintMeterPlus:OnInterfaceMenuListHasLoaded()
		Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "SprintMeterPlus", {"SprintMeterPlusMenuClick", "", "CRB_CompanionsSprites:btnFlyingNormal"})
end

function SprintMeterPlus:OnSprintMeterPlusMenuClick()
	self:OnSprintMeterPlusOn()
end
-----------------------------------------------------------------------------------------------
-- SprintMeterPlus Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/smp"
function SprintMeterPlus:OnSprintMeterPlusOn()
	self.bNotActive = false
	if 	self.PopUp:IsShown() then 
		self:OnButtonClosed()

	else
	self.PopUp:Show(true)
	self.wndMain:Show(true)
	self.wndMain:SetStyle("Moveable", true)
	self.wndMain:SetStyle("IgnoreMouse", false)
	end
end

function SprintMeterPlus:OnButtonClosed()
	self.wndMain:SetStyle("Moveable", false)
	self.wndMain:SetStyle("IgnoreMouse", true)
	self.PopUp:Close()
end



---------------------------------------------------------------------------------------------------
-- PopUp Functions
---------------------------------------------------------------------------------------------------

function SprintMeterPlus:OnScaleSliderChange( wndHandler, wndControl, fNewValue, fOldValue, bOkToChange )

		local fScale = fNewValue / 100
		self.wndMain:SetScale(fScale)
		self.PopUp:FindChild("MeterScaleValue"):SetText(fScale)
		self.tSettings.fScale = fScale

end

function SprintMeterPlus:StylesButtonClicked( wndHandler, wndControl, eMouseButton )
		self.PopUp:FindChild("StylesWindow"):Show(true)
		self.PopUp:FindChild("OptionsWindow"):Show(false)
end


function SprintMeterPlus:OptionsButtonClicked( wndHandler, wndControl, eMouseButton )
		self.PopUp:FindChild("OptionsWindow"):Show(true)
		self.PopUp:FindChild("StylesWindow"):Show(false)
end


function SprintMeterPlus:ChangeStyle( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )
	local style = wndHandler:GetName()

	if style == "SlimMeter" or "KuronaMeter" or "SprintMeter" then
		local l,t,r,b = self.wndMain:GetAnchorOffsets()

		local oldMain = self.wndMain
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, style .."Form", FixedHudStratumLow, self)

	
		local nHeight = self.wndMain:GetHeight()
		local nWidth = self.wndMain:GetWidth()

		self.wndMain:SetAnchorOffsets(l,t,l+250*self.tSettings.fWidth,t+nHeight)
		
		self.wndMain:Show(true)
		self.wndMain:SetStyle("Moveable", true)
		self.wndMain:SetStyle("IgnoreMouse", false)
		self.wndMain:SetScale(self.tSettings.fScale)
		self.tSettings.Style = style
		oldMain:Destroy()
		self:SetupFrameAlias()
		self.LastRun =0
		self.LastEvade1 = 0
		self.LastEvade2 = 0
		self.HazardWindow = self.wndMain:FindChild("HazardWindow")
	end
end


function SprintMeterPlus:OnWidthSliderChange( wndHandler, wndControl, fNewValue, fOldValue )
		self.tSettings.fWidth = fNewValue / 100
		self.PopUp:FindChild("WidthScaleValue"):SetText(self.tSettings.fWidth)
		local l,t,r,b = self.wndMain:GetAnchorOffsets()
		self.wndMain:SetAnchorOffsets(l,t,l+250*self.tSettings.fWidth,b)
end

-----------------------------------------------------------------------------------------------
-- SprintMeterPlus Instance
-----------------------------------------------------------------------------------------------
local SprintMeterPlusInst = SprintMeterPlus:new()
SprintMeterPlusInst:Init()



function SprintMeterPlus:OnFrame()
	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		return
	end
	
	self:DoHazards()
	
	local bOptionsOpen self.PopUp:IsShown()
	
	local nRunCurr = unitPlayer:GetResource(0)
	local nRunMax = unitPlayer:GetMaxResource(0)
	local nEvadeCurr = unitPlayer:GetResource(7)
	local nEvadeMax = unitPlayer:GetMaxResource(7)
	if not bOptionsOpen and not self.bActiveHazard then
		if nRunCurr == nRunMax and nEvadeCurr == nEvadeMax and self.bNotActive then return end
	end
	
	self.bNotActive = false
	
	local bWndVisible = self.wndMain:IsVisible()
	local bRunAtMax = nRunCurr == nRunMax or unitPlayer:IsDead()
	local nRunPerc = math.floor((nRunCurr / nRunMax)*100)
	local nEvadeHalf = nEvadeMax / 2
	local bEvadeAtMax = nEvadeCurr == nEvadeMax or unitPlayer:IsDead()
	
	
	if not self.PopUp:IsShown()	then
		if bWndVisible and bRunAtMax and bEvadeAtMax and not self.bJustFilled then
			self.bJustFilled = true
			Apollo.StopTimer("SprintMeterGracePeriod")
			Apollo.CreateTimer("SprintMeterGracePeriod", 0.5, false)
		end

			
		if not bRunAtMax or not bEvadeAtMax or self.bActiveHazard then
			self.bJustFilled = false
			Apollo.StopTimer("SprintMeterGracePeriod")
			self.wndMain:Show(true)
		end
	end
		
	
	if self.LastRun ~= nRunCurr then
		self.FrameAlias.ProgressBar:SetMax(nRunMax)
		self.FrameAlias.Percentage:SetText(nRunPerc.."%")
		self.FrameAlias.ProgressBar:SetProgress(nRunCurr, bWndVisible and nRunMax or 0)
	end
	
	local evade1 = nEvadeCurr / nEvadeHalf
	local evade2 = (nEvadeCurr-nEvadeHalf) / nEvadeHalf
		
	if self.LastEvade1 ~= evade1 then
		self.FrameAlias.DashProgressBar1:SetProgress(evade1)
	end
		
	if self.LastEvade2 ~= evade1 then
		self.FrameAlias.DashProgressBar2:SetProgress(evade2)
	end
		
	self.LastRun = nRunCurr
	self.LastEvade1 = evade1
	self.LastEvade2 = evade2
end


function SprintMeterPlus:OnSprintMeterGracePeriod()
	Apollo.StopTimer("SprintMeterGracePeriod")
	
	
	if not self.bActiveHazard then
		self.bJustFilled = false
		if self.wndMain and self.wndMain:IsValid() then
			self.wndMain:Show(false)
			self.bNotActive = true
		end
	else
		Apollo.CreateTimer("SprintMeterGracePeriod", 0.5, false)
	end
	
end



function SprintMeterPlus:OnSave(eLevel)
	
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	local l,t,r,b = self.wndMain:GetAnchorOffsets()
	local tsave = {}
	tsave.cSprintMeterWindowL = l
	tsave.cSprintMeterWindowT = t
	tsave.cSprintMeterWindowR = r
	tsave.cSprintMeterWindowB = b
	tsave.fScale = self.tSettings.fScale
	tsave.fWidth = self.tSettings.fWidth
	tsave.Style = self.tSettings.Style
	return tsave
end



function SprintMeterPlus:OnRestore(eLevel, table)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	self.tSettings = {}			
	
	for k,v in pairs(table) do
	   	self.tSettings[k]=v
	end

	
	if self.tSettings.cSprintMeterWindowL == nil then
		self.tSettings.cSprintMeterWindowL = -125
	end
	
	if self.tSettings.cSprintMeterWindowT == nil then
		self.tSettings.cSprintMeterWindowT = -154
	end
		
		
	if self.tSettings.fWidth == nil then
		self.tSettings.fWidth = 1
	end
	
	if self.tSettings.fScale == nil then
		self.tSettings.fScale = 1
	end
	if self.tSettings.Style == nil then
			self.tSettings.Style ="Normal"
	end
	
end


function SprintMeterPlus:SetDefaults()

	local screenH, screenW = Apollo.GetScreenSize()
	
		
		self.tSettings.fWidth = 1
		self.tSettings.fScale = 1

		
		local oldMain = self.wndMain		
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SprintMeterForm", FixedHudStratumLow, self)

		self.wndMain:SetAnchorOffsets(-125,-154,125,-63)

	
		self.wndMain:SetScale(1)
		

		self.PopUp:FindChild("MeterScaleValue"):SetText(self.tSettings.fScale)
		self.PopUp:FindChild("WidthScaleValue"):SetText(self.tSettings.fWidth)

		self.PopUp:FindChild("ScaleSliderBar"):SetValue(self.tSettings.fScale*100)
		self.PopUp:FindChild("WidthSliderBar"):SetValue(self.tSettings.fWidth*100)


		self.wndMain:Show(true)
		self.wndMain:SetStyle("Moveable", true)
		self.wndMain:SetStyle("IgnoreMouse", false)
		self.tSettings.Style ="SprintMeter"
	
		oldMain:Destroy()
		self:SetupFrameAlias()
		self.LastRun =0
		self.LastEvade1 = 0
		self.LastEvade2 = 0

end


function SprintMeterPlus:SetupFrameAlias()
	self.FrameAlias = {
	Dash1Lit = self.wndMain:FindChild("Dash1Lit"),
	Dash2Lit = self.wndMain:FindChild("Dash2Lit"),
	DashProgressBar1 = self.wndMain:FindChild("DashProgressBar1"),
	DashProgressBar2 = self.wndMain:FindChild("DashProgressBar2"),
	ProgressBar = self.wndMain:FindChild("ProgressBar"),
	Percentage = self.wndMain:FindChild("Percentage"),
}
end


function SprintMeterPlus:DoHazards()
	local hazardvalue = 0
	local strTooltip = ""
	self.bActiveHazard = false
	for idx, tActiveHazard in ipairs(HazardsLib.GetHazardActiveList()) do
		if tActiveHazard.eHazardType == HazardsLib.HazardType_Radiation then
			if tActiveHazard.fMeterValue then
				local perc = math.floor((tActiveHazard.fMeterValue / tActiveHazard.fMaxValue) * 100)
				self.bActiveHazard = true
				self.HazardWindow:SetText("Hazard: "..perc)
			end
		end
		if tActiveHazard.eHazardType == HazardsLib.HazardType_Temperature then
			if tActiveHazard.fMeterValue then
				local perc = math.floor((tActiveHazard.fMeterValue / tActiveHazard.fMaxValue) * 100)
				self.bActiveHazard = true
				self.HazardWindow:SetText("Heat: "..perc)
			end
		end
	end
	if self.nBreath < 100 then	
		self.HazardWindow:SetText("Breath: "..self.nBreath)
		self.bActiveHazard = true
	end	
	if not self.bActiveHazard then
		self.HazardWindow:SetText("")
	end
end


function SprintMeterPlus:OnBreathChanged(nBreath)
	self.nBreath = nBreath
end


